﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// サムネイルを表示する
    /// </summary>
    public partial class ThumbView : UserControl
    {
        #region Variables
        const int _borderIncrement = 2;
        Size _thumnailSize;        
        DocumentPaginator _paginator;
        #endregion

        #region Properties
        /// <summary>
        /// サムネイルの大きさ
        /// </summary>
        public Size ThumnailSize
        {
            get { return _thumnailSize; }
            set
            {
                if (null != value && _thumnailSize != value)
                {
                    _thumnailSize = value;
                    redraw();
                }
            }
        }
        #endregion

        #region Constructor
        public ThumbView()
        {
            InitializeComponent();

            _thumnailSize = new Size(100, 150);
            this.DataContextChanged += new DependencyPropertyChangedEventHandler(ThumbView_DataContextChanged);
        }
        #endregion

        #region Event handler
        void ThumbView_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            // DataContent instance check
            if (null == DataContext)
            {
                _listBox.Items.Clear();
            }

            // DataContent instance type check
            if (!(DataContext is DocumentPaginator))
            {
                TextBlock ermsg = new TextBlock(new Run(String.Format("Error! DataContext is an instance of an unexpected type : {0}.", DataContext.GetType())));
                _listBox.Items.Clear();
                _listBox.Items.Add(ermsg);
            }

            // Paginator instance check
            if (_paginator == (DocumentPaginator)DataContext)
                return;

            // update thumbnails
            _paginator = (DocumentPaginator)DataContext;
            redraw();
        }

        /// <summary>
        /// Page thumnail loadイベント，ページ詳細を書き込む
        /// </summary>
        void OuterBorder_Loaded(object sender, RoutedEventArgs e)
        {
            if (null == sender || !(sender is Border))
                return;
            
            Border brd = (Border)sender;
            if(brd.Child == null)
                fillPage(brd);
        }
        #endregion

        #region Private methods
        /// <summary>
        /// 表示更新
        /// </summary>
        void redraw()
        {
            this._listBox.Items.Clear();
            for (int index = 0; index < _paginator.PageCount; index++)
                _listBox.Items.Add(createEmptyBorder(index));
        }

        /// <summary>
        /// 描画領域を確保
        /// </summary>
        Border createEmptyBorder(int pageIndex)
        {
            Border OuterBorder          = new Border();
            OuterBorder.BorderThickness = new Thickness(2);
            OuterBorder.Margin     = new Thickness(3);
            OuterBorder.Width      = _thumnailSize.Width  + _borderIncrement;
            OuterBorder.Height     = _thumnailSize.Height + _borderIncrement + 30;
            OuterBorder.Background = Brushes.White;
            OuterBorder.Tag        = pageIndex;
            OuterBorder.Loaded += new RoutedEventHandler(OuterBorder_Loaded); // Load時にサムネイルを描画する

            return OuterBorder;
        }

        /// <summary>
        /// ページを描画
        /// </summary>        
        void fillPage(Border brd)
        {            
            int pageIndex = (int)brd.Tag;
            using (DocumentPage docPage = _paginator.GetPage(pageIndex))
                brd.Child = createPageThumb(docPage.Visual, pageIndex);

            // debug
            Console.WriteLine(String.Format("Filling page {0}.", pageIndex));
        }

        StackPanel createPageThumb(Visual visual, int pageIndex)
        {

            // ページ内容をVisualBrushに描画
            
            VisualBrush vb = new VisualBrush(visual);
            vb.ViewboxUnits = BrushMappingMode.Absolute;
            vb.Viewbox = new Rect(_paginator.PageSize);
            
            /*
            RenderTargetBitmap bitmap = new RenderTargetBitmap((int)_paginator.PageSize.Width, (int)_paginator.PageSize.Height , 96.0, 96.0, PixelFormats.Pbgra32);            
            bitmap.Render(visual);            
            */

            StackPanel sp = new StackPanel();

            Rectangle rect = new Rectangle();
            rect.BeginInit();
            rect.Width  = _thumnailSize.Width;
            rect.Height = _thumnailSize.Height;            
            rect.Fill   = vb;
            rect.EndInit();

            /*
            Image img  = new Image();
            img.Source = bitmap;
            img.Width  = _thumnailSize.Width;
            */
            
            TextBlock tb = new TextBlock(new Run((pageIndex + 1).ToString()));
            tb.FontSize  = 16;
            tb.HorizontalAlignment = HorizontalAlignment.Center;

            sp.Orientation = Orientation.Vertical;
            //sp.Children.Add(img);
            sp.Children.Add(rect);
            sp.Children.Add(tb);

            return sp;
        }
        #endregion
    }
}
