﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Net;
using System.Printing;

using System.IO;
using System.IO.Packaging;

using System.Windows.Annotations;
using System.Windows.Annotations.Storage;

using System.Windows.Xps;
using System.Windows.Xps.Packaging;

using WinForms = System.Windows.Forms;

namespace WpfApplication1
{
    /// <summary>
    /// Window1.xaml の相互作用ロジック
    /// </summary>
    public partial class Window1 : Window
    {
        #region Variables
        Uri _packageUri;
        string _xpsDocumentpath;
        Package _xpsPackage;
        XpsDocument _xpsDocument;
        AnnotationService _annotService;
        FileStream _annotStream;
        XmlStreamStore _annotStore;
        string _xpsFile = string.Empty;

        readonly string _annotStorePath = @"annotations.xml";
        readonly string _fixedDocumentSequenceContentType = "application/vnd.ms-package.xps-fixeddocumentsequence+xml";
        #endregion

        public Window1()
        {
            InitializeComponent();
            //Console.WriteLine(pluginFileFilter());            
        }

        /*
        string pluginFileFilter()
        {
            SerializerProvider serProv = new SerializerProvider();
            string filter = string.Empty;
            foreach (SerializerDescriptor sdesc in serProv.InstalledSerializers)
            {
                if (sdesc.IsLoadable)
                {
                    if (filter.Length > 0) filter += "|";
                    filter += sdesc.DisplayName + "(*" +
                        sdesc.DefaultFileExtension + ")|*" +
                        sdesc.DefaultFileExtension;
                }
            }

            return filter;
        }
        */

        private void OnOpen(object sender, ExecutedRoutedEventArgs e)
        {
            if (null != _xpsPackage)
            {
                string msg = "The currently open document needs to be closed before\nopening a new document. Ok to close the current document?";
                MessageBoxResult res = MessageBox.Show(msg, "Close current document?",
                    MessageBoxButton.OKCancel, MessageBoxImage.Question);
                if (res != MessageBoxResult.OK)
                    return;

                CloseDocument();                
            }

            WinForms.OpenFileDialog dialog = new System.Windows.Forms.OpenFileDialog();
            dialog.InitialDirectory = GetContentFolder();
            dialog.CheckFileExists = true;
            dialog.Filter = "XPS Document files (*.xps)|*xps";

            if (dialog.ShowDialog() == WinForms.DialogResult.OK)
                OpenDocument(dialog.FileName);
        }

        bool OpenDocument(string fileName)
        {
            _xpsDocumentpath = fileName;
            _packageUri = new Uri(fileName, UriKind.Absolute);
            try
            {
                _xpsDocument = new XpsDocument(fileName, FileAccess.Read);
            }
            catch (System.UnauthorizedAccessException)
            {
                string msg = fileName + "\n\nThe specified file is read-only which prevents storing user annotations.";
                MessageBox.Show(msg, "Read-only file", MessageBoxButton.OKCancel, MessageBoxImage.Error);
                return false;
            }

            _xpsPackage = PackageStore.GetPackage(_packageUri);
            if ((null == _xpsPackage) || (null == _xpsDocument))
            {
                MessageBox.Show("Unnable to get package from file.");
                return false;
            }

            FixedDocumentSequence fds = _xpsDocument.GetFixedDocumentSequence();
            if (null == fds)
            {
                string msg = fileName + "\n\nThe document package within the specified file does not contain a FixedDocumentSequence.";
                MessageBox.Show(msg, "Package Error");
                return false;
            }

            _docViewer.Document = fds;

            // thumbview
            _thumbView.DataContext = fds.DocumentPaginator;
            // debug

            /* Visual 描画 文字列を書くだけ，結果はOK．
            FormattedText ftxt = new FormattedText("ABC",
                new System.Globalization.CultureInfo("en-us"),
                FlowDirection.LeftToRight, new Typeface(this.FontFamily,
                    FontStyles.Normal, FontWeights.Normal, new FontStretch()),
                    this.FontSize, this.Foreground);
            DrawingVisual drawingvisual = new DrawingVisual();
            DrawingContext drawingcontext = drawingvisual.RenderOpen();
            drawingcontext.DrawText(ftxt, new Point(2,2));
            drawingcontext.Close();

            RenderTargetBitmap bitmap = new RenderTargetBitmap(180, 180, 120, 96, PixelFormats.Pbgra32);
            bitmap.Render(drawingvisual);
            _image.Source = bitmap;
             */
            /*
            DocumentPage dp = fds.DocumentPaginator.GetPage(0);
            RenderTargetBitmap bitmap = new RenderTargetBitmap((int)dp.Size.Width,(int)dp.Size.Height, 96.0, 96.0, PixelFormats.Pbgra32);
            bitmap.Render(dp.Visual);
            _image.Source = bitmap;
             * */
            /*
            DocumentPage dp  = fds.DocumentPaginator.GetPage(0);
            RenderTargetBitmap bitmap = new RenderTargetBitmap(60,90, 96.0, 96.0, PixelFormats.Pbgra32);
            bitmap.Render(dp.Visual);
            _image.Source = bitmap;
            _image.Height = 90;
            _image.Width  = 60;
             */
            /*
            RenderTargetBitmap bitmap = new RenderTargetBitmap((int)rect.Width, (int)rect.Height, 96.0, 96.0, PixelFormats.Pbgra32);
            bitmap.Render(visual);

            StackPanel sp = new StackPanel();
            Image img = new Image();
            img.Source = bitmap;
            img.Height = size.Height;
             */

            _menuFileClose.IsEnabled = true;
            _menuFilePrint.IsEnabled = true;
            _menuViewAnnotations.IsEnabled = true;
            _menuViewIncreaseZoom.IsEnabled = true;
            _menuViewDecreaseZoom.IsEnabled = true;

            Uri fixedDocumentSeqUri = GetFixedDocumentSequenceUri();
            if (_menuViewAnnotations.IsChecked)
                StartAnnotation();                

            _docViewer.Focus();

            return true;
        }

        Uri GetFixedDocumentSequenceUri()
        {
            foreach (PackagePart part in _xpsPackage.GetParts())
            {
                if (part.ContentType == _fixedDocumentSequenceContentType)
                    return part.Uri;
            }
            return null;
        }

        string GetContentFolder()
        {
            return Directory.GetCurrentDirectory();
        }

        Package getPackage(string fileName)
        {
            Package inputPackage = null;

            WebRequest wreq = System.Net.WebRequest.Create(fileName);
            if (null != wreq)
            {
                WebResponse wres = wreq.GetResponse();
                if (null != wres)
                {
                    Stream str = wres.GetResponseStream();
                    if (null != str)
                    {
                        inputPackage = Package.Open(str);
                    }
                }
            }

            return inputPackage;
        }

        void StartAnnotation()
        {
            if (null == _annotService)
                _annotService = new AnnotationService(_docViewer);

            if (_annotService.IsEnabled == true)
                _annotService.Disable();

            _annotStream = new FileStream(_annotStorePath, FileMode.OpenOrCreate, FileAccess.ReadWrite);

            _annotStore = new XmlStreamStore(_annotStream);

            _annotService.Enable(_annotStore);
        }

        void StopAnnotation()
        {
            if (null != _annotStore)
            {
                _annotStore.Flush();
                _annotStream.Flush();
                _annotStream.Close();
                _annotStore = null;
            }

            if (_annotService != null)
            {
                if (_annotService.IsEnabled)
                {
                    _annotService.Disable();
                    _annotService = null;
                }
            }
        }

        void OnExit(object sender, EventArgs e)       
        {
            Close();
        }

        void OnClosed(object sender, EventArgs e)
        {
            CloseDocument();
        }

        void OnClose(object target, ExecutedRoutedEventArgs args)
        {
            CloseDocument();
        }

        void CloseDocument()
        {
            StopAnnotation();
            _docViewer.Document = null;
            if (null != _xpsPackage)
            {
                _xpsPackage.Close();
                _xpsPackage = null;
            }
            if (null != _packageUri)
                PackageStore.RemovePackage(_packageUri);

            _menuFileClose.IsEnabled = false;
            _menuFilePrint.IsEnabled = false;
            _menuViewAnnotations.IsEnabled = false;
            _menuViewDecreaseZoom.IsEnabled = false;
            _menuViewIncreaseZoom.IsEnabled = false;
        }

        void OnPrint(Object sender, ExecutedRoutedEventArgs args)
        {
            PrintDocument();
        }

        void PrintDocument()
        {
            if (null == _docViewer)
                return;

            if (_menuViewAnnotations.IsCheckable == false)
                _docViewer.Print();
            else
            {
                PrintDialog prntDialog = new PrintDialog();
                if ((bool)prntDialog.ShowDialog())
                {
                    string docDir = Directory.GetCurrentDirectory();
                    XpsDocumentWriter xdw = PrintQueue.CreateXpsDocumentWriter(prntDialog.PrintQueue);
                    try
                    {
                        xdw.Write(GetAnnotationDocumentPaginator(_xpsDocument.GetFixedDocumentSequence()));
                    }
                    catch (PrintingCanceledException)
                    { }
                    Directory.SetCurrentDirectory(docDir);
                }
            }
        }

        void ShowAnnotations(Object sender, EventArgs e)
        {
            if (_menuViewAnnotations.IsEnabled && _menuViewAnnotations.IsCheckable)
                StartAnnotation();
        }

        void HideAnnotations(object sender, EventArgs e)
        {
            StopAnnotation();
        }

        AnnotationDocumentPaginator GetAnnotationDocumentPaginator(
            FixedDocumentSequence fds)
        {
            return new AnnotationDocumentPaginator(fds.DocumentPaginator, _annotStore);
        }

    }
}
